/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*!
    \file Bookmarks.h
    \brief Bookmarks functionality - marking words as favorite, managing marked
    words, searching in marked words (with cached translations)

    \author Bartosz Szatkowski <bulislaw@linux.com>
*/

#ifndef BOOKMARKS_H
#define BOOKMARKS_H

#include <QtSql>
#include <QString>
#include <QVariant>
#include <QStringList>
#include <QList>
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QSqlError>
#include <QDir>
#include <QDebug>
#include "../../include/settings.h"
#include "../../include/translation.h"
#include "../../include/AccentsNormalizer.h"
class BookmarkTranslation;


/*!
    Bookmarks are a way to store words that You think You will need to search
    for often.

    When You add a bookmark (by clicking on "star" in words list) You add it to
    a special list with cached translations from all available dictionaries so
    You can search for them quickly even when You delete corresponding dict.
*/
class Bookmarks : public AccentsNormalizer {
public:
    Bookmarks();
    ~Bookmarks();

    /*!
        Adds new word and translation to bookmarks
        \param translation new translation to be saved and cached as a bookmark
    */
    void add(Translation* translation);

    void add(QString key,QString removeAccentKey,QString value);

    /*!
        Removes word and corresponding translation cache from bookmarks list
        \param translation translation to be removed
    */
    void remove(Translation* translation);

    /*!
        \return all bookmarks (word and translation as a translation object
        as a list)
    */
    QList<Translation*> list();

    /*!
        Searches in bookmarks for a given word (wildcards may apply '*' and '?')
        \param word word to search for
        \return list of matching Translation objects
    */
    QList<Translation*> searchWordList(QString word);

    /*!
        Searches for final translation of a given word
        \return word translation list in text format xml to be formatted
        and displayed
        \param word word to search for
    */
    QStringList search(QString word, QString dbname);

    /*!
        Clears bookmarks database
    */
    void clear();

    /*!
        \return true if a given word is already in bookmarks
        \param word word to check
    */
    bool inBookmarks(QString word);

private:
    bool checkAndCreateDb(QString dbName = "");

    
    QString dbName;
    QSqlDatabase getDbCnx();
    QMap<QChar, QRegExp> letters;
    QRegExp noLetter;
    QSqlDatabase mdb;

};

#endif // BOOKMARKS_H
