/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file DictManagerWidget.h
    \brief Implements dictionaries management widget

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#ifndef DICTMANAGERWIDGET_H
#define DICTMANAGERWIDGET_H

#include <QWidget>
#include <QtGui>
#include "../../include/GUIInterface.h"
#include "MenuWidget.h"
#include "DictManagerModel.h"
#include <QtDeclarative/QDeclarativeView>
#include <QtDeclarative/QDeclarativeContext>
#include <QDebug>


/*!
  Shows list of loaded dictionaries and their states (active/inactive).
  It allows to change dicts states, add a new dict, remove selected one or
  change settings of selected dict. All changes are saved automatically after
  hiding of this widget.
*/
class DictManagerWidget : public QDialog {
    Q_OBJECT
public:
    //! Constructor
    /*!
      \param parent parent of this widget, which must be a subclass of
      GUIInterface, because it will use it to get info about loaded plugins
      and dicts.
    */
    explicit DictManagerWidget(GUIInterface *parent = 0);

protected:
    void showEvent(QShowEvent *e);
    void hideEvent(QHideEvent *e);
    void keyPressEvent(QKeyEvent *e);

Q_SIGNALS:
    //! Emitted when hiding widget, it will save states of dictionaries
    /*!
        \param list of only active dictionaries
    */
    void selectedDictionaries(QList<CommonDictInterface*>);

    //! Emitted when user wants to add a new dictionary
    /*!
        \param new dictionary returned by specific plugin dialog
    */
    void addDictionary(CommonDictInterface*);

    //! Emitted when user wants to remove a dictionary
    /*!
        \param dictionary which will be removed
    */
    void removeDictionary(CommonDictInterface*);


public Q_SLOTS:

    #ifndef Q_WS_MAEMO_5
        void save();
    #endif

private Q_SLOTS:
    /*! Shows "plugin select" dialog and then plugin specific "add a new dictionary"
        dialog, which will return new CommonDictInterface* object, which is
        later passed as a parameter of addDictionary signal*/
    void addNewDictButtonClicked();

    /*! Passes dictionary selected from a list as a parameter of removeDictionary
        signal */
    void removeButtonClicked();

    //! user selects one of items
    void itemSelected(QListWidgetItem*);

    //! Shows plugin's settings dialog
    void settingsButtonClicked();

    //! Each change of state (that needs to be saved) should call this to
    //! indicate state change
    void changed();

    //! saves changes
    void saveChanges();

private:
    void initalizeUI();
    QPushButton* addNewDictButton;
    QPushButton* removeDictButton;
    QPushButton* settingsButton;

    QVBoxLayout* verticalLayout;
    QHBoxLayout* buttonGroup;
    QListWidget* dictList;

    //holds association between items on list and CommonDictInterface objects
    QHash<QListWidgetItem*, CommonDictInterface*> dictsHash;
    GUIInterface* guiInterface;

    #ifndef Q_WS_MAEMO_5
    QDeclarativeView* qmlView;
    QDeclarativeContext* ctxt;
    DictManagerModel* model;
    #endif

    bool _changed;

    void refreshDictsList();

    #ifndef Q_WS_MAEMO_5
        QPushButton* closeButton;
        bool _save;
    #endif
};

#endif // DICTMANAGERWIDGET_H
