/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file DictManagerWidget.cpp
    \brief Implements applications settings widget
    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "SettingsWidget.h"
#include <QDebug>

SettingsWidget::SettingsWidget(GUIInterface *parent) :
        QDialog(parent)
{
    guiInterface = parent;

    setWindowTitle(tr("Settings"));

    initalizeUI();

    setModal(true);
}

void SettingsWidget::initalizeUI() {
    verticalLayout = new QVBoxLayout;
    setLayout(verticalLayout);

    historySizeLayout = new QHBoxLayout;

    historySizeSpinBox = new SpinBox;
    historySizeSpinBox->setMinimum(1);
    historySizeSpinBox->setMaximum(50);

    historySizeToolTip = tr("Limits maximum number of words saved in history");

    #ifdef Q_WS_MAEMO_5
        historySizeInfoToolButton = new QToolButton;
        historySizeInfoToolButton->setIcon(QIcon::fromTheme("general_information"));
    #endif

    historySizeLayout->addWidget(historySizeSpinBox);
    #ifdef Q_WS_MAEMO_5
        historySizeLayout->addWidget(historySizeInfoToolButton);
    #endif




    searchResultLayout = new QHBoxLayout;

    searchResultSizeSpinBox = new SpinBox;
    searchResultSizeSpinBox->setMinimum(0);
    searchResultSizeSpinBox->setMaximum(500);
    searchResultSizeSpinBox->setSpecialValueText(tr("Unlimited"));

    searchResultSizeToolTip = tr("Limits maximum number of found words, affects"
                                 " only when searching in file.");

    #ifdef Q_WS_MAEMO_5
        searchResultSizeInfoToolButton = new QToolButton;
        searchResultSizeInfoToolButton->setIcon(QIcon::fromTheme("general_information"));
    #endif

    searchResultLayout->addWidget(searchResultSizeSpinBox);
    #ifdef Q_WS_MAEMO_5
        searchResultLayout->addWidget(searchResultSizeInfoToolButton);
    #endif





    spinBoxesFormLayout = new QFormLayout;
    spinBoxesFormLayout->addRow(tr("Search result size"),
                                searchResultLayout);
    spinBoxesFormLayout->addRow(tr("History size"),
                                historySizeLayout);




    verticalLayout->addLayout(spinBoxesFormLayout);


    checkBoxesLabel = new QLabel(tr("Search in:"),this);

    searchInBookmarksCheckBox = new QCheckBox(tr("Bookmarks"));
    searchInDictionariesCheckBox = new QCheckBox(tr("Dictionaries"));

    verticalLayout->addSpacing(20);
    verticalLayout->addWidget(checkBoxesLabel);
    verticalLayout->addWidget(searchInDictionariesCheckBox);
    verticalLayout->addWidget(searchInBookmarksCheckBox);


    connect(historySizeSpinBox, SIGNAL(valueChanged(int)), this,
            SLOT(changed()));
    connect(searchResultSizeSpinBox, SIGNAL(valueChanged(int)), this,
            SLOT(changed()));
    connect(searchInDictionariesCheckBox, SIGNAL(toggled(bool)), this,
            SLOT(changed()));
    connect(searchInBookmarksCheckBox, SIGNAL(toggled(bool)), this,
            SLOT(changed()));


#ifdef Q_WS_MAEMO_5
        connect(searchResultSizeInfoToolButton, SIGNAL(clicked()),
                this, SLOT(showSearchResultSizeInfo()));

        connect(historySizeInfoToolButton, SIGNAL(clicked()),
                this, SLOT(showHistorySizeInfo()));
#else
        historySizeSpinBox->setToolTip(historySizeToolTip);
        searchResultSizeSpinBox->setToolTip(searchResultSizeToolTip);
#endif


    settings = 0;

    #ifndef Q_WS_MAEMO_5
        setMinimumWidth(250);
        setMaximumWidth(250);
        footerLayout = new QHBoxLayout;
        closeButton = new QPushButton(tr("Save"));
        footerLayout->addStretch(0);
        footerLayout->addWidget(closeButton);
        verticalLayout->addLayout(footerLayout);

        setMinimumHeight(sizeHint().height());
        setMaximumHeight(sizeHint().height());
        connect(closeButton, SIGNAL(clicked()), this, SLOT(save()));
    #endif
}

void SettingsWidget::showEvent(QShowEvent *e) {

   #ifndef Q_WS_MAEMO_5
       _save = false;
   #endif

   settings = guiInterface->settings();

   historySizeSpinBox->setValue(
            settings->value("history_size").toInt());

    searchResultSizeSpinBox->setValue(
            settings->value("search_limit").toInt());

    if(settings->value("search_bookmarks") == "true")
        searchInBookmarksCheckBox->setChecked(true);
    else
        searchInBookmarksCheckBox->setChecked(false);

    if(settings->value("search_dictionaries") == "true")
        searchInDictionariesCheckBox->setChecked(true);
    else
        searchInDictionariesCheckBox->setChecked(false);

    _changed = false;
    QDialog::showEvent(e);
}

void SettingsWidget::hideEvent(QHideEvent *e) {
    QDialog::hideEvent(e);

    #ifndef Q_WS_MAEMO_5
        if(settings && _save) {
    #else
        if(settings && _changed &&
           QMessageBox::question(this,
                                 tr("Save"),
                                 tr("Do you want to save changes?"),
             QMessageBox::Save, QMessageBox::Cancel) == QMessageBox::Save) {

    #endif
        Settings* newSettings = new Settings;
        newSettings->setValue("history_size",
                              QString::number(historySizeSpinBox->value()));
        newSettings->setValue("search_limit",
                              QString::number(
                                      searchResultSizeSpinBox->value()));

        if(searchInDictionariesCheckBox->isChecked())
            newSettings->setValue("search_dictionaries", "true");
        else
            newSettings->setValue("search_dictionaries", "false");

        if(searchInBookmarksCheckBox->isChecked())
            newSettings->setValue("search_bookmarks", "true");
        else
            newSettings->setValue("search_bookmarks", "false");

        //setting new settings only if they are different than old ones
        QString key;
        foreach(key, newSettings->keys()) {
            if(settings->value(key) != newSettings->value(key)) {
                guiInterface->setSettings(newSettings);
                break;
            }
        }

    }
    if(settings) {
        delete settings;
        settings = 0;
    }
    _changed = false;
}


void SettingsWidget::changed() {
    _changed = true;
}

#ifndef Q_WS_MAEMO_5
    void SettingsWidget::save() {
        _save = true;
        hide();
    }
#endif


#ifdef Q_WS_MAEMO_5
    void SettingsWidget::showHistorySizeInfo() {
        Q_EMIT notify(Notify::Warning, historySizeToolTip);
    }

    void SettingsWidget::showSearchResultSizeInfo() {
        Q_EMIT notify(Notify::Warning, searchResultSizeToolTip);
    }
#endif
