
/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file WordListwidget.cpp
    \brief Displays list of words found in dictionaries

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "WordListWidget.h"
#include "WordListProxyStyle.h"
#include "../../include/translation.h"
#include <QKeyEvent>


WordListWidget::WordListWidget(QWidget *parent):
    QTreeView(parent) {

    //creating new model to store words and stars
    model = new QStandardItemModel(this);
    setModel(model);
    setHeaderHidden(true);
    setRootIsDecorated(false);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    //set our custom style to draw checkboxes as stars
    proxyStyle = new WordListProxyStyle();
    setStyle(proxyStyle);

    //setting size of star in pixels, on maemo checboxes are much bigger
    #ifdef Q_WS_MAEMO_5
        checkBoxWidth = 70;
    #else
        checkBoxWidth = 25;
    #endif
}


WordListWidget::~WordListWidget() {
    if(proxyStyle)
        delete proxyStyle;
}

void WordListWidget::addWord(QString word, int row) {
    QStandardItem* item = new QStandardItem(word);

    //we don't want to allow user to edit word
    item->setFlags(item->flags() ^ Qt::ItemIsEditable);

    QStandardItem* itemCheckBox = new QStandardItem();
    //creating checkbox item
    itemCheckBox->setFlags((itemCheckBox->flags() ^ Qt::ItemIsEditable) |
                           Qt::ItemIsUserCheckable);

    /*checking if word is already in bookmarks, information about that is
    stored in its translation object (not all translations have to be in
    bookmarks)*/
    bool bookmark = false;
    Translation* t;
    foreach(t, searchResult[word]) {
        if(t->isBookmark()) {
            bookmark = true;
            break;
        }
    }

    if(bookmark)
        itemCheckBox->setCheckState(Qt::Checked);
    else
        itemCheckBox->setCheckState(Qt::Unchecked);

    //add item to model
    model->setItem(row,0, item);
    model->setItem(row,1, itemCheckBox);
}


void WordListWidget::showSearchResults(
        QHash<QString, QList<Translation *> > result) {

    clear();
    searchResult = result;

    if(searchResult.count()>0) {
        setEnabled(true);
        model->setColumnCount(2);
        model->setRowCount(result.count());

        int row=0;
        QHash<QString, QList<Translation*> >::iterator i;
        for(i = searchResult.begin(); i != searchResult.end(); i++) {
               addWord(i.key(), row++);
        }

        model->sort(0);
        resizeColumns();
    }
    else {
        QStandardItem* item = new QStandardItem(tr("Can't find any matching words"));
        item->setFlags(item->flags() ^ Qt::ItemIsEditable);
        item->setTextAlignment(Qt::AlignCenter);
        setEnabled(false);

        model->setItem(0,item);
    }

    setFocus();
}

void WordListWidget::wordClicked(QModelIndex index) {
    //we're getting translation based on data in index
    Q_EMIT showTranslation(
            searchResult[index.data().toString()]);
}

void WordListWidget::wordChecked(QModelIndex index) {

    //save new item state
    Qt::CheckState state =
            Qt::CheckState(index.data(Qt::CheckStateRole).toInt());


    //getting index of item which contains word which should be added/removed
    //from bookmarks
    QModelIndex item = selectedIndexes().at(0);
    if(!item.isValid()) return;

    //to shorten lag between clicking on a star and its change
    repaint();

    //depending on new state emit suitable signal
    if(state == Qt::Checked) {
        Q_EMIT addBookmark(searchResult[item.data().toString()]);
    }
    else {
        Q_EMIT removeBookmark(searchResult[item.data().toString()]);

        Translation* t;
        bool onlyBookmarks = true;
        foreach(t, searchResult[item.data().toString()]) {
            if(t->isBookmark() == 1 || t->isBookmark()==0) {

                onlyBookmarks = false;
                t->setBookmark(0);
            }
            else {
                searchResult[item.data().toString()].removeAt(searchResult[item.data().toString()].indexOf(t));
            }
        }

        if(onlyBookmarks) {
            searchResult.remove(item.data().toString());
            model->removeRow(item.row());
        }
    }
}


void WordListWidget::mouseReleaseEvent(QMouseEvent *event) {

    //firstly we normally handle this event
    QTreeView::mouseReleaseEvent(event);

    //then we check at which item user clicked
    QModelIndex index = indexAt(event->pos());
    if(!index.isValid()) return;

    /*if there are no selected items we return, that occurs sometimes
    on maemo, when user is scrolling list and clicks to stop the scroll,
    system doesn't select item but emits mouseReleaseEvent*/
    if(selectedIndexes().count() == 0) return;

    //if user doesn't click either on a word or on a star, return
    if(selectedIndexes().at(0) != index && selectedIndexes().at(1) != index)
        return;

    int c = index.column();
    if(c==0)
        //if column is 0 user clicked on a word
        wordClicked(index);
    else
        //else user clicked on a star
        wordChecked(index);
}

void WordListWidget::resizeEvent(QResizeEvent *event) {
    resizeColumns();
    QTreeView::resizeEvent(event);
}

void WordListWidget::resizeColumns() {
    setColumnWidth(0, viewport()->width() -checkBoxWidth - 5);
    setColumnWidth(1, checkBoxWidth);
}

void WordListWidget::keyPressEvent(QKeyEvent *event) {
    QTreeView::keyPressEvent(event);

    if(event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) {
        if(selectedIndexes().count() == 0) return;

        wordClicked(selectedIndexes().at(0));
    }
}

void WordListWidget::lockList() {
    setEnabled(false);
}

void WordListWidget::unlockList() {
    setEnabled(true);
}

void WordListWidget::clear() {
    model->clear();

    QHash<QString, QList<Translation*> >::iterator i;
    for(i = searchResult.begin(); i != searchResult.end(); i++) {
           Translation*t;
           foreach(t, i.value()) {
               delete t;
           }
    }
    searchResult.clear();
}
