/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
     \file StarDictPlugin.h
     \brief Implementation of stardict plugin's main class.

     \author Jakub Jaszczynski
*/

#ifndef STARDICTPLUGIN_H
#define STARDICTPLUGIN_H


#include <QObject>
#include <QDialog>
#include <QRegExp>
#include <QTime>
#include <QFile>
//#include <QXmlStreamReader>
#include <QtPlugin>
#include <QHash>
#include <QIcon>
#include <QtEndian>

#include "../../include/CommonDictInterface.h"
#include "../../include/settings.h"
#include "StarDictDialog.h"
#include "TranslationStarDict.h"

class TranslationXdxf;

class StarDictPlugin : public CommonDictInterface
{
    Q_OBJECT
    Q_INTERFACES(CommonDictInterface)
public:
    StarDictPlugin(QObject *parent=0);

    ~StarDictPlugin();

    //! \returns source language code iso 639-2
    QString langFrom() const;

    //! \returns destination language code iso 639-2
    QString langTo() const;

    //! \returns dictionary name (like "old English" or so)
    QString name() const;

    //! \returns dictionary type (xdxf, google,starDict translate, etc)
    QString type() const;

    //! returns information about dictionary in xml (name, authors, etc)
    QString infoNote() const;

    /*! \returns DictDialog object that creates dialogs
        for adding a new dictionary and changing plugin settings
    */
    DictDialog* dictDialog();

    //! \returns new, clean copy of plugin with settings set as in Settings*
    CommonDictInterface* getNew(const Settings*) const;

    //! \returns whether plugin can start searching
    bool isAvailable() const;

    //! \returns a description of a word given by a QString
    QString search(QString key) {
        return search(key, 0, 0);
    }

    /*!
        \return a description of a word given by a QString
        \param offset offset of translation to be cut out
        \param len lenght of translation to be cut out
    */
    QString search(QString key, qint64 offset, qint32 len);

    //! \returns current plugin settings
    Settings* settings();

    //! Sets new settings
    bool setSettings(const Settings*);

    //! \returns plugin icon
    QIcon* icon();

    //! \returns plugin icon's resource path
    QString iconPath();

    /*!
        plugin should delete any files (eg. cache) that have been created and are ready
        to be deleted
    */
    void clean() {;}



public Q_SLOTS:
    /*!
        performs search in a dictionary
        \param  word word to search for in a dictionary
        \param limit limit on number of results

        After finishing search it has to emit
        \see CommonDictInterface:finalTranslation  finalTranslation
    */
    QList<Translation*> searchWordList(QString word, int limit=0);

    //! stops current operation
    void stop();

    //! loads translations for each plugin only once
    void retranslate();


private:

    /*!
        searches for a list of words similar to a word in file
        \param word key compared with keys in a file
        \param limit limits the number of translations in returned list,
        0 means unlimited
        \returns list of translations
    */
    QList<Translation*> searchWordListCache(QString word, int limit=0);

    /*!
        searches for a list of words similar to a word in a starDict file
        \param word key compared with keys in a starDict file
        \param limit limits the number of translations in returned list,
        0 means unlimited
        \returns list of translations
    */
    QList<Translation*> searchWordListFile(QString word, int limit=0);

    /*!
        searches for a translation of a word which is exactly like a key
        in a starDict file
    */
    QString searchFile(QString key);

    //! scans dictionary file to get information about it
    bool getDictionaryInfo();

    /*!
        Reads and process (converting to qstring) data from StarDict dictionary
        file (*.dict[.dz])
        \return converted translation
        \param QByteArray raw data to process
        \param mode StarDict parametr "sametypesequence"
    */
    QString format(QByteArray, QString mode,QString key);

    /*!
        Reads bytes bytes of data or reads until \0
        \param it iterator to given data
        \param end end of data
        \param bytes to read
        \return readed data chunk
    */
    QByteArray read(QByteArray::iterator it, QByteArray::iterator end,
           int bytes = 0);

    /*!
        Interpret data basis on mode (StarDict dict data type)
        \param it iterator on given data set
        \param end iterator pointing to the data end
        \param mode stardict dict data type
        \param last used to interpret sametypesequence field last letter (see
        StarDict format description)
        \return QSting containing interpreted data chunk
    */
    QString interpret(QByteArray::iterator it, QByteArray::iterator end,
            QChar mode,QString key, bool last = false);

    QString _langFrom;
    QString _langTo;
    QString _name;
    QString _infoNote;
    QIcon _icon;
    //! Path to icon
    QString _iconPath;
    volatile bool stopped;
    Settings *_settings;
    StarDictDialog* _dictDialog;
    Settings* _ifoFileSettings;

    friend class StarDictTests;
};

#endif // XDXFPLUGIN_H


