/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*! \file tst_backbone.cpp
\brief Backbone tests

\author Bartosz Szatkowski <bulislaw@linux.com>
*/


#include <QtCore/QString>
#include <QtTest/QtTest>
#include <QStringList>
#include <QList>
#include <QTime>
#include "../../src/mdictionary/backbone/backbone.h"
#include "CommonDictInterfaceMock.h"

class BackboneTest : public QObject
{
    Q_OBJECT

    QList<CommonDictInterface*> dict;
    int total;
    Backbone* back;
    void addDicts();

public:
    BackboneTest();

private Q_SLOTS:
    void init();
    void cleanup();
    void addDictionaryTest();
    void removeDictionaryTest();
    void selectedDictionaryTest();
    void stopSearchingTest();
    void searchTest();
    void translationTest();
    void historyTest();
    void quitTest();
};

BackboneTest::BackboneTest()
{
    total = 5;
    for(int i = 0; i < total; i++)
        dict.push_back(new CommonDictInterfaceMock());
    QApplication app(0, 0);
}



void BackboneTest::init()
{
    for(int i = 0; i < total; i++) {
        dict[i] = new CommonDictInterfaceMock();
        CommonDictInterfaceMock * cd = (CommonDictInterfaceMock*) dict[i];
        cd->tov =   QString("to%1").arg(i);
        cd->fromv = QString("from%1").arg(i);
        cd->namev = QString("name%1").arg(i);
        cd->typev = QString("type%1").arg(i);
    }
    back = new Backbone("empty","empty", 1);
    addDicts();
}

void BackboneTest::addDicts() {
    for(int i = 0; i < total; i++) {
        back->addInternalDictionary(dict[i],true);
    }
}



void BackboneTest::cleanup()
{
    delete back;
}




void BackboneTest::addDictionaryTest()
{
    QCOMPARE(back->getDictionaries().size(), total);
    for(int i = 0; i < total; i++)
        QCOMPARE(back->getDictionaries().keys().contains(dict[i]), QBool(true));
}

void BackboneTest::removeDictionaryTest() {
    for(int i = 0; i < total-1; i++)
        back->removeDictionary(dict[i]);

    QVERIFY2(back->getDictionaries().contains(dict[total-1]) == 1,
             "Deleted wrong dictionaries");

}



void BackboneTest::selectedDictionaryTest() {
    QList<CommonDictInterface* > selected;
    back->selectedDictionaries(selected);
    foreach(bool d, back->getDictionaries().values())
        QCOMPARE(d, false);

    selected << dict[0] << dict[4];

    back->selectedDictionaries(selected);
    foreach(CommonDictInterface* d, back->getDictionaries().keys())
        if(selected.contains(d))
            QCOMPARE(back->getDictionaries()[d], true);
        else
            QCOMPARE(back->getDictionaries()[d], false);
}




void BackboneTest::stopSearchingTest() {
    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*) dict[i];
        m->stopped = 0;
        back->addDictionary(dict[i]);
    }

    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*)dict[i];
        QCOMPARE(m->stopped, 0);
    }

    back->search("");
    back->stopSearching();


    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*)dict[i];
        QCOMPARE(m->stopped, 1);
    }


}




void BackboneTest::searchTest() {
    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*) dict[i];
        m->stopped = 1;
        back->addDictionary(dict[i]);
    }

    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*)dict[i];
        QCOMPARE(m->stopped, 1);
    }

    QString list("pigwa");
    back->_searchBookmarks = 0;
    back->_searchDicts = 1;

    back->search(list);
    sleep(1);


    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*)dict[i];
        QCOMPARE(m->stopped, 0);
    }
}




void BackboneTest::translationTest() {
    QSignalSpy translatS(back, SIGNAL(ready()));
    QVERIFY2 (translatS.isValid() == true, "ready() signal is invalid");


    QTime time;
    time.start();
    QString list("nic");
    back->_searchBookmarks = 0;
    back->_searchDicts = 1;
    back->search(list);
    sleep(2);
    qDebug() << back->result().size() << total;
    back->translationReady();
    qDebug() << back->result().size() << total;

    QVERIFY2(back->result().size() == total*2, "Lost some of the translations");
}

void BackboneTest::quitTest() {
    QSignalSpy translatS(back, SIGNAL(closeOk()));
    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*) dict[i];
        m->stopped = 0;
        back->addDictionary(dict[i]);
    }

    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*)dict[i];
        QCOMPARE(m->stopped, 0);
    }

    back->search("");
    back->quit();

    for(int i = 0; i < total; i++) {
        CommonDictInterfaceMock *m = (CommonDictInterfaceMock*)dict[i];
        QCOMPARE(m->stopped, 1);
    }
    QVERIFY2(translatS.count() == 1, "Lost finall 'closeOk()' signal");
}

void BackboneTest::historyTest() {
    History* history = new History(10,this);

    history->add("mleko");

    QCOMPARE(history->nextAvailable(), FALSE);
    QCOMPARE(history->prevAvailable(), FALSE);
    QCOMPARE(history->listAvailable(), TRUE);


    history->add("szklanka");
    QCOMPARE(history->nextAvailable(), FALSE);
    QCOMPARE(history->prevAvailable(), TRUE);

    history->previous();
    QCOMPARE(history->nextAvailable(), TRUE);
    QCOMPARE(history->prevAvailable(), FALSE);

    history->add("pic");
    QStringList words = history->list();
    QCOMPARE(words[0], QString("pic"));
    QCOMPARE(words[1], QString("mleko"));

    history->add("qqq");

    history->previous();
    history->add("pic");

    words = history->list();
    QCOMPARE(words[0], QString("qqq"));
    QCOMPARE(words[1], QString("pic"));
    QCOMPARE(words[2], QString("mleko"));

    history->setCurrentElement(0);
    QCOMPARE(history->nextAvailable(), FALSE);
    QCOMPARE(history->prevAvailable(), TRUE);
    QCOMPARE(history->listAvailable(), TRUE);

    history->setCurrentElement(1);
    QCOMPARE(history->nextAvailable(), TRUE);
    QCOMPARE(history->prevAvailable(), TRUE);
    QCOMPARE(history->listAvailable(), TRUE);

    history->setMaxSize(1);
    words = history->list();
    QCOMPARE(words[0], QString("pic"));

    QCOMPARE(history->nextAvailable(), FALSE);
    QCOMPARE(history->prevAvailable(), FALSE);
    QCOMPARE(history->listAvailable(), TRUE);
}


QTEST_APPLESS_MAIN(BackboneTest);

#include "tst_Backbone.moc"
